#! /usr/bin/perl
# $Id: 99_pod_coverage.t,v 1.2 2007/12/13 23:40:54 dk Exp $

use strict;
use warnings;

use Test::More tests => 1; ok(1); # XXX later

# eval 'use Test::Pod::Coverage';
# plan skip_all => 'Test::Pod::Coverage required for testing POD coverage'
#     if $@;
# all_pod_coverage_ok();
